﻿using UnityEngine;
using Microsoft.ML.Probabilistic;
using Microsoft.ML.Probabilistic.Models;
using Microsoft.ML.Probabilistic.Algorithms;

public class Main : MonoBehaviour {

	void Start () {
		TwoCoins ();
		ConstrainedGaussian ();
		ConstrainedGaussianWithVariableThreshold ();
	}

	void TwoCoins () {
		Variable<bool> firstCoin = Variable.Bernoulli (0.5);
		Variable<bool> secondCoin = Variable.Bernoulli (0.5);
		Variable<bool> bothHeads = firstCoin & secondCoin;

		InferenceEngine engine = new InferenceEngine ();

		Debug.Log ("Probability both coins are heads: " + engine.Infer (bothHeads) + "\n");

		bothHeads.ObservedValue = false;

		Debug.Log ("Probability distribution over firstCoin: " + engine.Infer (firstCoin) + "\n");
	}

	void ConstrainedGaussian () {
		Variable<double> x = Variable.GaussianFromMeanAndVariance (0, 1).Named ("x");
		Variable.ConstrainTrue (x > 0.5);

		InferenceEngine engine = new InferenceEngine ();
		engine.Algorithm = new ExpectationPropagation ();

		Debug.Log ("Dist over x=" + engine.Infer (x) + "\n");
	}

	void ConstrainedGaussianWithVariableThreshold () {
		for (double thresh = 0; thresh <= 1; thresh += 0.1) {
			Variable<double> x = Variable.GaussianFromMeanAndVariance (0, 1).Named ("x");
			Variable.ConstrainTrue (x > thresh);

			InferenceEngine engine = new InferenceEngine ();
			engine.Algorithm = new ExpectationPropagation ();

			Debug.Log ("Dist over x given thresh of " + thresh + "=" + engine.Infer (x) + "\n");
		}
	}
}
